import java.awt.*;
import acm.program.*;
import acm.util.*;
import acm.graphics.*;

public class ImageProcessor extends GraphicsProgram {
	/**
	 * Creates a GImage of all possible combinations of red and blue.
	 * 
	 * @return A GImage of all possible combinations of red and blue.
	 */
	private GImage getRedBlueSpectrum() {
		/* Create a multidimensional array large enough to hold the result. */
		int[][] pixels = new int[256][256];
		
		/* Set all pixels based on their position. */
		for (int row = 0; row < pixels.length; row++) {
			for (int col = 0; col < pixels[row].length; col++) {
				pixels[row][col] = GImage.createRGBPixel(row, 0, col);
			}
		}
		
		return new GImage(pixels);
	}
	
	/**
	 * Given a GImage, returns a new GImage formed by disabling the color blue
	 * from that image.
	 * 
	 * @param image The image whose blue should be disabled.
	 * @return That image, with no blue remaining.
	 */
	private GImage disableBlue(GImage image) {
		/* Obtain the underlying pixel array. */
		int[][] pixels = image.getPixelArray();
		
		for (int row = 0; row < pixels.length; row++) {
			for (int col = 0; col < pixels[row].length; col++) {
				/* Extract the red and green components. */
				int red = GImage.getRed(pixels[row][col]);
				int green = GImage.getGreen(pixels[row][col]);
				
				/* Reconstruct the pixel with just those components. */
				pixels[row][col] = GImage.createRGBPixel(red, green, 0);
			}
		}
		
		return new GImage(pixels);
	}
	
	/**
	 * Cycles the red, green, and blue components of an image, returning the result.
	 * 
	 * @param image The image whose colors should be permuted.
	 * @return The resulting image.
	 */
	private GImage twistColors(GImage image) {
		int[][] pixels = image.getPixelArray();
		
		for (int row = 0; row < pixels.length; row++) {
			for (int col = 0; col < pixels[row].length; col++) {
				/* Extract red, green, and blue. */
				int red = GImage.getRed(pixels[row][col]);
				int green = GImage.getGreen(pixels[row][col]);
				int blue = GImage.getBlue(pixels[row][col]);
				
				/* Rotate those intensities. */
				pixels[row][col] = GImage.createRGBPixel(green, blue, red);
			}
		}
		
		return new GImage(pixels);
	}
	
	/**
	 * Converts an image to grayscale based on the perceived intensity of each pixel.
	 * This algortihm uses the formula 
	 * <p>
	 * perceivedIntensity = 0.3 * red + 0.59 * green + 0.11 * blue.
	 * 
	 * @param image The image to convert to grayscale.
	 * @return That image, in grayscale.
	 */
	private GImage toGrayscale(GImage image) {
		int[][] pixels = image.getPixelArray();
		
		for (int row = 0; row < pixels.length; row++) {
			for (int col = 0; col < pixels[row].length; col++) {
				int red = GImage.getRed(pixels[row][col]);
				int green = GImage.getGreen(pixels[row][col]);
				int blue = GImage.getBlue(pixels[row][col]);
				
				/* Compute the pixel's intensity. */
				int intensity = (int)(0.3 * red + 0.59 * green + 0.11 * blue);
				
				/* Construct a gray pixel with that intensity. */
				pixels[row][col] = GImage.createRGBPixel(intensity, intensity, intensity);
			}
		}
		
		return new GImage(pixels);
	}
	
	public void run() {
		GImage picture = new GImage("rainbow-dash.png");
		add(toGrayscale(picture));
	}
}
